@@ -1,5 +1,5 @@
 #
-# Copyright 2011-2018 Universidad Complutense de Madrid
+# Copyright 2011-2019 Universidad Complutense de Madrid
 #
 # This file is part of Numina
 #
@@ -14,7 +14,7 @@ import logging
 
 import numina.core.query
 import numina.core.deptree
-import numina.core.objimport
+import numina.util.objimport
 import numina.util.parser
 import numina.datamodel
 
@@ -44,7 +44,7 @@ class Pipeline(object):
     def _get_base_class(self, entry):
 
         recipe_fqn = entry['class']
-        return numina.core.objimport.import_object(recipe_fqn)
+        return numina.util.objimport.import_object(recipe_fqn)
 
     def _get_base_object(self, entry):
 
@@ -52,6 +52,9 @@ class Pipeline(object):
 
         args = entry.get('args', ())
         kwargs = entry.get('kwargs', {})
+        links = entry.get('links', {})
+        if links:
+            kwargs['query_options'] = links
 
         recipe = Cls.__new__(Cls, *args, **kwargs)
         recipe.__init__(*args, **kwargs)
@@ -193,7 +196,7 @@ class InstrumentDRP(object):
        pipeline : dict of Pipeline
 
     """
-    def __init__(self, name, configurations, modes, pipelines, products=None, datamodel=None):
+    def __init__(self, name, configurations, modes, pipelines, products=None, datamodel=None, version='undefined'):
         self.name = name
         self.configurations = configurations
         self.modes = modes
@@ -202,6 +205,7 @@ class InstrumentDRP(object):
             self.datamodel = datamodel()
         else:
             self.datamodel = numina.datamodel.DataModel()
+        self.version = version
 
     def query_provides(self, product, pipeline='default', search=False):
         """Return the mode that provides a given product"""
@@ -213,7 +217,7 @@ class InstrumentDRP(object):
         try:
             return pipe.who_provides(product)
         except KeyError:
-            raise ValueError('no mode provides %s' % product)
+            raise ValueError(f'no mode provides {product}')
 
     def search_mode_provides(self, product, pipeline='default'):
         """Search the mode that provides a given product"""
@@ -224,7 +228,7 @@ class InstrumentDRP(object):
             if obj.name() == product:
                 return ProductEntry(obj.name(), mode.key, field)
         else:
-            raise ValueError('no mode provides %s' % product)
+            raise ValueError(f'no mode provides {product}')
 
     def iterate_mode_provides(self, modes, pipeline):
         """Return the mode that provides a given product"""
@@ -236,27 +240,29 @@ class InstrumentDRP(object):
                     if provide.type.isproduct():
                         yield provide.type, mode, key
             except KeyError:
-                warnings.warn('Mode {} has not recipe'.format(mode_key))
+                warnings.warn(f'Mode {mode_key} has not recipe')
 
     def configuration_selector(self, obsres):
         warnings.warn("configuration_selector is deprecated, use 'select_configuration' instead",
-                      DeprecationWarning)
-        return self.select_configuration(obsres)
+                      DeprecationWarning, stacklevel=2)
+        return self.select_configuration_old(obsres)
 
     def product_label(self, tipo):
         return tipo.name()
 
-    def select_configuration(self, obresult):
+    def select_configuration_old(self, obresult):
         """Select instrument configuration based on OB"""
 
         logger = logging.getLogger(__name__)
         logger.debug('calling default configuration selector')
 
         # get first possible image
-        ref = obresult.get_sample_frame()
+        ref_frame = obresult.get_sample_frame()
+        ref = ref_frame.open()
+        extr = self.datamodel.extractor_map['fits']
         if ref:
             # get INSCONF configuration
-            result = self.datamodel.extractor.extract('insconf', ref)
+            result = extr.extract('insconf', ref)
             if result:
                 # found the keyword, try to match
                 logger.debug('found insconf config uuid=%s', result)
@@ -269,10 +275,10 @@ class InstrumentDRP(object):
                         if conf.name == result:
                             return conf
                     else:
-                        raise KeyError('insconf {} does not match any config'.format(result))
+                        raise KeyError(f'insconf {result} does not match any config')
 
             # If not, try to match by DATE
-            date_obs = self.datamodel.extractor.extract('observation_date', ref)
+            date_obs = extr.extract('observation_date', ref)
             for key, conf in self.configurations.items():
                 if key == 'default':
                     # skip default
@@ -288,6 +294,46 @@ class InstrumentDRP(object):
             logger.debug('no match, using default configuration')
             return self.configurations['default']
 
+    def select_configuration(self, obresult):
+        return self.select_profile(obresult)
+
+    def select_profile(self, obresult):
+        """Select instrument profile based on OB"""
+
+        logger = logging.getLogger(__name__)
+        logger.debug('calling default profile selector')
+        # check configuration
+        insconf = obresult.configuration
+        if insconf != 'default':
+            key = insconf
+            date_obs = None
+            keyname = 'uuid'
+        else:
+            # get first possible image
+            sample_frame = obresult.get_sample_frame()
+            if sample_frame is None:
+                key = obresult.instrument
+                date_obs = None
+                keyname = 'name'
+            else:
+                return self.select_profile_image(sample_frame.open())
+        return key, date_obs, keyname
+
+    def select_profile_image(self, img):
+        """Select instrument profile based on FITS"""
+
+        extr = self.datamodel.extractor_map['fits']
+
+        date_obs = extr.extract('observation_date', img)
+        key = extr.extract('insconf', img)
+        if key is not None:
+            keyname = 'uuid'
+        else:
+            key = extr.extract('instrument', img)
+            keyname = 'name'
+
+        return key, date_obs, keyname
+
     def get_recipe_object(self, mode_name, pipeline_name='default'):
         """Build a recipe object from a given mode name"""
         active_mode = self.modes[mode_name]
@@ -313,66 +359,20 @@ class ProductEntry(object):
         return msg
 
 
-class InstrumentConfiguration(object):
-
-    def __init__(self, instrument):
-        self.instrument = instrument
-        self.name = 'config'
-        self.uuid = ''
-        self.date_start = 0
-        self.date_end = 0
-        self.components = {}
-
-    def get(self, path, **kwds):
-        # split key
-        vals = path.split('.')
-        component = vals[0]
-        key = vals[1]
-        conf = self.components[component]
-        return conf.get(key, **kwds)
-
-
-class ConfigurationEntry(object):
-    def __init__(self, values, depends):
-        self.values = values
-        self.depends = depends
-
-    def get(self, **kwds):
-        result = self.values
-        for dep in self.depends:
-            key = kwds[dep]
-            result = result[key]
-        return result
-
-
-class ComponentConfigurations(object):
-
-    def __init__(self):
-        self.name = 'config'
-        self.uuid = ''
-        self.data_start = 0
-        self.data_end = 0
-        self.component = 'component'
-        self.configurations = {}
-
-    def get(self, key, **kwds):
-        conf = self.configurations[key]
-        return conf.get(**kwds)
-
-
 class ObservingMode(object):
     """Observing modes of an Instrument."""
-    def __init__(self):
+    def __init__(self, instrument=''):
         self.name = ''
         self.key = ''
-        self.instrument = ''
+        self.instrument = instrument
         self.summary = ''
         self.description = ''
         self.tagger = None
         self.validator = None
         self.build_ob_options = None
+        self.rawimage = None
 
-    def validate(self):
+    def validate(self, obsres):
         return True
 
     def build_ob(self, partial_ob, backend, options=None):
@@ -390,8 +390,10 @@ class ObservingMode(object):
 
     def tag_ob(self, partial):
         if self.tagger is not None:
+            warnings.warn("per mode taggers are deprecated, recipe requirements provide al required informatation",
+                          DeprecationWarning, stacklevel=2)
             partial.tags = self.tagger(partial)
         return partial
 
     def __repr__(self):
-        return "ObservingMode(name={})".format(self.name)
+        return f"ObservingMode(name={self.name})"
