@@ -407,22 +407,21 @@ static PyMethodDef module_functions[] = {
     { NULL, NULL, 0, NULL } /* sentinel */
 };
 
-#if PY_MAJOR_VERSION >= 3
-    static struct PyModuleDef moduledef = {
-        PyModuleDef_HEAD_INIT,
-        "_combine",     /* m_name */
-        combine__doc__,  /* m_doc */
-        -1,                  /* m_size */
-        module_functions,    /* m_methods */
-        NULL,                /* m_reload */
-        NULL,                /* m_traverse */
-        NULL,                /* m_clear */
-        NULL,                /* m_free */
-    };
-#endif
-#if PY_MAJOR_VERSION >= 3
-  PyMODINIT_FUNC PyInit__combine(void)
-  {
+
+static struct PyModuleDef moduledef = {
+    PyModuleDef_HEAD_INIT,
+    "_combine",     /* m_name */
+    combine__doc__,  /* m_doc */
+    -1,                  /* m_size */
+    module_functions,    /* m_methods */
+    NULL,                /* m_reload */
+    NULL,                /* m_traverse */
+    NULL,                /* m_clear */
+    NULL,                /* m_free */
+};
+
+PyMODINIT_FUNC PyInit__combine(void)
+{
    PyObject *m;
    m = PyModule_Create(&moduledef);
    if (m == NULL)
@@ -440,26 +439,4 @@ static PyMethodDef module_functions[] = {
    Py_INCREF(CombineError);
    PyModule_AddObject(m, "CombineError", CombineError);
    return m;
-  }
-#else
-PyMODINIT_FUNC init_combine(void)
-{
-  PyObject *m;
-  m = Py_InitModule3("_combine", module_functions, combine__doc__);
-  import_array();
-
-  if (m == NULL)
-    return;
-
-  if (CombineError == NULL)
-  {
-    /*
-     * A different base class can be used as base of the exception
-     * passing something instead of NULL
-     */
-    CombineError = PyErr_NewException("_combine.CombineError", NULL, NULL);
-  }
-  Py_INCREF(CombineError);
-  PyModule_AddObject(m, "CombineError", CombineError);
 }
-#endif
